/* -------------------------------------------------------------------------------------------------------------------------
* Project           :C-to-FPGA Solution for Acclerating Tomographic Reconstruction
* File Name         :structs.h (from project UW Emission Reconstruction) 
*                    Copyright (c) 2004 University of Washington - Imaging Research Laboratory. All rights reserved.
* Version           :1.0 (V1)
* (V1)Author        :Adam Alessio
* (V1)Created       :Wed, Jan 14 2004
* References        :
* Description       :Contains declaration of all structures
*
******This file may not be distributed or modified without this header*****
*-------------------------------------------------------------------------------------------------------------------------*/

#ifndef STRUCTS_H
#define STRUCTS_H

#include "allocate.h"
#include "log.h"
#include "SwapEndian.h"

#define RAMP_FILTER      0   // For FBP filter choice
#define GHAMMING_FILTER  1 

struct sino_info{
  int    nProjs;
  int    nAngles;
  int    nPlanes;
  int    nFrames;
  int sinoSize;
  double   projSpacing;
  double   angleSpacing;
  double   planeSpacing;
  double projWidth;
  float   intensityScale;
  float**** sinogramData;
  float**** VarSinogramData;
  float*** AWsinogramData;
  float **CompleteForwardProj;
  //  float ****varianceData;
  // float** timeData;
  // char*   daqInfo;
  // char*   sortInfo;
};

struct recon_image_info{
  int xDim;
  int yDim;
  int zDim;
  int nFrames;
  double tfov;
  double afov;
  int nSubSets;
  double xPixelSize;
  double yPixelSize;
  double zPixelSize;
  double intensityScale;
  float ****imageData;
  float ****anatomicalLabels;
};

struct exec_parameters{
  int num_iterations;
  int num_subsets;
  char interpolate_flag;
  char recon_mask_flag;
  int reconType;
  double tfov;
  double afov;
  double sliceThick;
  short fbp_type;
  double fbp_cutoff;
  int resolution;
  int ReconPlanes[2];
  double projWidthMultiple;
  double p;          // prior shape parameter
  double globalBeta; // prior scaling
  short penalty_type;
  char anatomical_labels_flag;
  char AW_flag;   // AW for stat recon, can not have regular attenCorrect and this...
  char OutEachIt;  //Output image at every iteration (and subset)
  char attenCorrect_flag;
  char *AnatomicalLabelName;
  char	*InputSinogramFileName;
  char	*InitialImageFileName;
  char	*OutputImageFileName;
  char *PSFfileName;
  char *AttenCorrectFileName;
  char *config_fname;
};

struct fbp_filter{
  short   type;
  double  cutoff;
  double* filter;
};

struct recon_data{
  float **Y;
  float **X;
  float **AW;
  short **recon_mask;
};

struct p_column_dbl{
  int nRows;
  double *val;
  int *row;
};

struct p_column_flt{
  int nRows;
  double norm;
  float *val;
  int *row;
};

struct system_matrix_info{
  int srfDim[3];
  short systemResponse_flag;
  float ***srf;
  struct p_column_flt P_j;
  struct p_column_flt ***p_matrix;
};

struct filter{
  short   type;
  double  cutoff;
  double* filter;
};

struct ICDprior{
  double NX[6];
  double DX[4];
  double DX3D[8];
  double NX3D[8];
  double CX;
  float pNX[6];  /* Info for anatomical labels based prior */
  float pDX[4];
  float pDX3D[8];
  float pNX3D[8];
  double p;
  double BetaSX;
  double D1,D2;
};


struct all_data{  /* all data except transmission attenuation sino */
double **X; /* recon image, initially CBP */
float **Y; /* sinog. counts */
  float **AW;
#ifdef CORR_SEP
float **Y_sep;   /* 2nd sinogram separated for rebinned data */
#endif
int **mask;
double scatter;
struct sino_info sino;
int Nsup; /* nb pixels w/ mask not zero */
int **recon_mask;
double mk; 
int currentReconPlane;
int currentReconFrame;
#ifdef EM_PML
double bk;
#endif
#ifdef EM_RECON
float **em_norm;  
#endif
};

struct neighbors{
	double NX[4]; /* hor and vert neighbors */
	double DX[4]; /* diagonal neighbors */	
};

struct neighbors3D{
	double NX[6]; /* hor and vert neighbors */
	double DX[4]; /* diagonal neighbors */
	double NX3D[8];
	double DX3D[8];
};



struct ICMdata{  /* parameters computed during ICM */
double D1,D2; /* derivatives */
#if defined AXIALCOR || defined AXIALMEAN || defined MAP3D
struct neighbors3D neigh;
#else
struct neighbors neigh; 
#endif 
double CX; /* center pixel */
double ML; 
double SX; /* old est. sigma^p */
double P;
};


struct A_correction{
	float ***Y_t; /* forward projected transmission attenuation sino    */
	char mode;   /* 'e' for emission, 't' for transmission */
};

struct N_correction{
	float ***Yn;  /* Normalization Sinogram for all planes*/
	int Norm_FLAG;
};

struct D_correction{
	float ***Yd;
	int Dead_FLAG;
};

struct all_corrections{
	struct A_correction A_cor;
	struct N_correction N_cor;
	struct D_correction D_cor;
};



struct correlated_info{
	float **Z;
	float *Q;
	float *Zn;
	double *ZiS;
	double *ZiPn;
	int *neighInd;
	struct neighbors neigh; 
	int tester;
	float **cov;
};

struct axialMean_info{
	float **HH;
	int **HHindex;
	int NumCor;
};

#endif
